%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Task Force on Modern Heuristic Optimization Test Beds
% Working Group on Modern Heuristic Optimization
% Intelligent Systems Subcommittee
% Power System Analysis, Computing, and Economic Committee
%
% 2018 Grid Optimization Competition
% Evaluating the Performance of Modern Heuristic Optimizers on Stochastic Optimization Problems applied to Smart Grids
% 
% Test bed B:
% Dynamic OPF in Presence of Renewable Energy and Electric Vehicles 
%
% Based on 2014 and 2017 competition AND [1], [2]
%
% [1] J. Arvalo, F. Santos, S. Rivera, Uncertainty cost functions for solar photovoltaic generation, wind
% energy generation, and plug-in electric vehicles: Mathematical expected value and verification by monte
% carlo simulation, International Journal of Power and Energy Conversion, in press, 2018.
% http://www.inderscience.com/info/ingeneral/forthcoming.php?jcode=ijpec
% [2] S. Rivera, A. Romero, Dispatch Modeling Incorporating Maneuver Components, Wind Power and
% Electric Vehicles Penetration, using Heuristic Techniques, Application of Modern Heuristic Optimization
% Techniques in Power and Energy Systems, ed: Wiley, in press, 2018.
%
% Questions can be directed to Sergio Rivera srriverar@unal.edu.co
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% A set of Montecarlo Simulations VS Solar Analytical Uncertainty Cost Function 
clc
close all
                    %% SOLAR PARAMETERS                    
                    
                    Ws=100; % Scheduled Power
                    Psr= 100; 
                    Gstd= 775; 
                    Rc=116;  
                    Wmax= 150; 
                    lambda= 6;  
                    beta= 0.25;
                    mu=lambda;  
                    sigma=beta;

N=100000;         % The cases 1, 2 and 3 in the competition uses only 2000

Cu=300;  
Co=700;   
    
ii=1;
iii=1;

CO=[];
CU=[];
W=[];

%% A set of montecarlo simulations
tic  
for i=1:N

    G(i)=lognrnd(mu,sigma);
      
    if ((G(i)<Rc)&&(G(i)>0))
        W(i)=Psr*(G(i)^2/(Gstd*Rc));
    elseif G(i)>Rc
        W(i)=Psr*(G(i)/Gstd);
    end
    
       if Ws>W(i)
          CO(ii)=Co*(Ws-W(i));
          ii=ii+1;
       else
             CU(iii)=Cu*(W(i)-Ws);
             iii=iii+1;
       end
end
   
   figure(1)
   
   subplot(4,1,1)
   %title('Irradiance, Power, Costo due to Underestimate and Cost due to overestimate Histograms','color','Black','fontsize',10); 
   hist(G)
   grid on;
   %title('Irradiance Histogram','color','Black','fontsize',11);
   title('Histograms','color','Black','fontsize',12);
   ylabel('Scenarios','color','Black','fontsize',9); 
   xlabel('Irradiance [W/m^2]','color','Black','fontsize',9);
   subplot(4,1,2)
   hist(W)
   grid on;
   %title('Power Histogram','color','Black','fontsize',11);
   ylabel('Scenarios','color','Black','fontsize',9); 
   xlabel('Power [MW]','color','Black','fontsize',9);
   subplot(4,1,3)
   hist(CU)
   grid on;
   %title('Cost due to underestimate Histogram','color','Black','fontsize',11);
   ylabel('Scenarios','color','Black','fontsize',9); 
   xlabel('Cost due to underestimate [$]','color','Black','fontsize',9);
   subplot(4,1,4)
   hist(CO)
   grid on;
   %title('Cost due to overestimate Histogram','color','Black','fontsize',11);
   ylabel('Scenarios','color','Black','fontsize',9);
   xlabel('Cost due to overestimate [$]','color','Black','fontsize',9);
   
   
   xo=mean(CO);
   xu=mean(CU);
      
   figure(2)
   CUCO=[CU CO];
   hist(CUCO)
   title( 'UCF Histogram','color','Black','fontsize',12) , grid on;
   ylabel('Scenarios','color','Black','fontsize',10); 
   xlabel('Penalty cost (UCF) [$]','color','Black','fontsize',11);
   ExpectedUncertaintyCost_Montecarlo=mean(CUCO)
   toc
%% Analytical Caculation

   Wrc=(Psr*Rc)/Gstd;

   if Wrc<Ws 
     
   ua2=(log(Ws*Gstd/Psr)-lambda)/(sqrt(2)*beta);
   ub2=(log(Wmax*Gstd/Psr)-lambda)/(sqrt(2)*beta);
   int2_s= (Cu/2)*( (Psr*exp(((beta^2)/2)+lambda)/(Gstd))*(erf(ub2-(beta/sqrt(2))) - erf(ua2-(beta/sqrt(2))) ) -Ws *(erf(ub2)-erf(ua2)));% subestimado [42]

   Cas=int2_s; 
   else
    
   ua1=(log(sqrt(Ws*Gstd*Rc/Psr))-lambda)/(sqrt(2)*beta);
   ub1=(log(sqrt(Wrc*Gstd*Rc/Psr))-lambda)/(sqrt(2)*beta);
   
   ua2=(log(Wrc*Gstd/Psr)-lambda)/(sqrt(2)*beta);
   ub2=(log(Wmax*Gstd/Psr)-lambda)/(sqrt(2)*beta);
     
    int1_s=(Cu/2)*( (Psr*exp(2*beta^2+2*lambda)/(Gstd*Rc))* (erf(ub1-sqrt(2)*beta)-erf(ua1-sqrt(2)*beta))-Ws*(erf(ub1)-erf(ua1)));%Integral [46]
    int2_s= (Cu/2)*( (Psr*exp(((beta^2)/2)+lambda)/(Gstd))*(erf(ub2-(beta/sqrt(2))) - erf(ua2-(beta/sqrt(2))) ) -Ws *(erf(ub2)-erf(ua2)));%integral[42]-BIEN

   Cas=int1_s+int2_s;
  end

if Ws<Wrc
    ua3=-inf;
    ub3=(log(sqrt(Ws*Gstd/Psr))-lambda)/(sqrt(2)*beta);
    
   int1_o=(Co/2)*( Ws*(erf(ub3)-erf(ua3)) - (Psr*exp(2*beta^2+2*lambda)/(Gstd*Rc))* (erf(ub3-sqrt(2)*beta)-erf(ua3-sqrt(2)*beta)));% integral [46]
   Cao=int1_o;
else
    ua3=-inf;    
    ub3=(log(sqrt(Wrc*Gstd*Rc/Psr))-lambda)/(sqrt(2)*beta);

    ua4=(log(Wrc*Gstd/Psr)-lambda)/(sqrt(2)*beta);
    ub4=(log(Ws*Gstd/Psr)-lambda)/(sqrt(2)*beta);
        
    int1_o= (Co/2)*( Ws*(erf(ub3)-erf(ua3)) - (Psr*exp(2*beta^2+2*lambda)/(Gstd*Rc))* (erf(ub3-sqrt(2)*beta)-erf(ua3-sqrt(2)*beta)));% integral [46]-BIEN
  
    int2_o= (Co/2)*( Ws *(erf(ub4)-erf(ua4))-(Psr*exp(((beta^2)/2)+lambda)/(Gstd))*(erf(ub4-(beta/sqrt(2))) - erf(ua4-(beta/sqrt(2))) ));
    Cao=int1_o+int2_o;
end

ExpectedUncertaintyCost_Analytical=Cas+Cao
